#|
This file is part of Ordo.

Copyright (C) 2025 Ray Miller

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, version 3.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <https://www.gnu.org/licenses/>.
|#

(define-module (ordo inventory)
  #:use-module (ice-9 eval-string)
  #:use-module (ice-9 match)
  #:use-module (ice-9 textual-ports)
  #:use-module (oop goops)
  #:use-module ((ordo connection) #:select (local-connection))
  #:use-module (ordo logger)
  #:use-module (srfi srfi-1)
  #:export (<host>
            defhost
            host?
            host-name
            host-connection
            host-tags

            resolve-hosts
            load-inventory!))

(define *inventory* '())

(define-class <host> ()
  (name #:init-keyword #:name #:getter host-name)
  (connection #:init-keyword #:connection #:getter host-connection)
  (tags #:init-keyword #:tags #:getter host-tags))

(define (host? h) (is-a? h <host>))

(define (defhost name . args)
  (let ((host (apply make <host> #:name name args)))
    (set! *inventory* (cons host *inventory*))))

(define (load-inventory! filename)
  (log-msg 'INFO "Loading inventory " filename)
  (eval-string (call-with-input-file filename get-string-all)
               #:file filename)
  (when (null? *inventory*)
    (log-msg 'NOTICE "Inventory is empty, only localhost will be available")
    (defhost "localhost" #:connection (local-connection))))

(define (tagged-every? wanted-tags)
  (lambda (h)
    (lset= equal? wanted-tags (lset-intersection equal? (host-tags h) wanted-tags))))

(define (tagged-any? wanted-tags)
  (lambda (h)
    (not (null? (lset-intersection equal? (host-tags h) wanted-tags)))))

(define (named? hostname)
  (lambda (h)
    (string=? (host-name h) hostname)))

(define-method (resolve-hosts expr)
  (match expr
    ((? string? hostname) (filter (named? hostname) *inventory*))
    ('all *inventory*)
    (('tagged tag) (filter (tagged-every? (list tag)) *inventory*))
    (('tagged/every tag . tags) (filter (tagged-every? (cons tag tags)) *inventory*))
    (('tagged/any tag . tags) (filter (tagged-any? (cons tag tags)) *inventory*))
    ((. hostnames) (filter (lambda (h) (member (host-name h) hostnames string=?)) *inventory*))))
